"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _native = _interopRequireDefault(require("./native"));

var _instance = _interopRequireDefault(require("./instance"));

var _instantiateComponent = _interopRequireDefault(require("./instantiateComponent"));

var _getElementKeyName = _interopRequireDefault(require("./getElementKeyName"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

/**
 * Fragment Component
 */
var FragmentComponent =
/*#__PURE__*/
function (_NativeComponent) {
  _inheritsLoose(FragmentComponent, _NativeComponent);

  function FragmentComponent(element) {
    return _NativeComponent.call(this, element) || this;
  }

  var _proto = FragmentComponent.prototype;

  _proto.mountComponent = function mountComponent(parent, parentInstance, context, childMounter) {
    // Parent native element
    this._parent = parent;
    this._parentInstance = parentInstance;
    this._context = context;
    this._mountID = _host.default.mountID++;
    var instance = {
      _internal: this
    };
    this._instance = instance;
    var fragment = this.getNativeNode();
    var children = this._currentElement; // Process children

    this.mountChildren(children, context);

    if (childMounter) {
      childMounter(fragment, parent);
    } else {
      for (var i = 0; i < fragment.length; i++) {
        var child = fragment[i];

        _host.default.driver.appendChild(child, parent);
      }
    }

    return instance;
  };

  _proto.mountChildren = function mountChildren(children, context) {
    var _this = this;

    var renderedChildren = this._renderedChildren = {};
    var fragment = this.getNativeNode();
    var renderedChildrenImage = children.map(function (element, index) {
      var renderedChild = (0, _instantiateComponent.default)(element);
      var name = (0, _getElementKeyName.default)(renderedChildren, element, index);
      renderedChildren[name] = renderedChild;
      renderedChild._mountIndex = index; // Mount

      var mountImage = renderedChild.mountComponent(_this._parent, _this._instance, context, function (nativeNode) {
        if (Array.isArray(nativeNode)) {
          for (var i = 0; i < nativeNode.length; i++) {
            fragment.push(nativeNode[i]);
          }
        } else {
          fragment.push(nativeNode);
        }
      });
      return mountImage;
    });
    return renderedChildrenImage;
  };

  _proto.unmountComponent = function unmountComponent(notRemoveChild) {
    if (this._nativeNode) {
      _instance.default.remove(this._nativeNode);

      if (!notRemoveChild) {
        for (var i = 0; i < this._nativeNode.length; i++) {
          _host.default.driver.removeChild(this._nativeNode[i]);
        }
      }
    } // Do not need remove child when their parent is removed


    this.unmountChildren(true);
    this._currentElement = null;
    this._nativeNode = null;
    this._parent = null;
    this._parentInstance = null;
    this._context = null;
    this._instance = null;
  };

  _proto.updateComponent = function updateComponent(prevElement, nextElement, prevContext, nextContext) {
    // Replace current element
    this._currentElement = nextElement;
    this.updateChildren(this._currentElement, nextContext);
  };

  _proto.getNativeNode = function getNativeNode() {
    if (this._nativeNode == null) {
      this._nativeNode = [];
    }

    return this._nativeNode;
  };

  _proto.getPublicInstance = function getPublicInstance() {
    return this.getNativeNode();
  };

  _proto.getName = function getName() {
    return 'fragment';
  };

  return FragmentComponent;
}(_native.default);

var _default = FragmentComponent;
exports.default = _default;