"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _ref = _interopRequireDefault(require("./ref"));

var _instantiateComponent = _interopRequireDefault(require("./instantiateComponent"));

var _shouldUpdateComponent = _interopRequireDefault(require("./shouldUpdateComponent"));

var _getElementKeyName = _interopRequireDefault(require("./getElementKeyName"));

var _instance = _interopRequireDefault(require("./instance"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var STYLE = 'style';
var CHILDREN = 'children';
var TREE = 'tree';
var EVENT_PREFIX_REGEXP = /^on[A-Z]/;
/**
 * Native Component
 */

var NativeComponent =
/*#__PURE__*/
function () {
  function NativeComponent(element) {
    this._currentElement = element;
  }

  var _proto = NativeComponent.prototype;

  _proto.mountComponent = function mountComponent(parent, parentInstance, context, childMounter) {
    // Parent native element
    this._parent = parent;
    this._parentInstance = parentInstance;
    this._context = context;
    this._mountID = _host.default.mountID++;
    var props = this._currentElement.props;
    var type = this._currentElement.type;
    var instance = {
      _internal: this,
      type: type,
      props: props
    };
    var appendType = props.append || TREE; // Default is node

    this._instance = instance; // Clone a copy for style diff

    this._prevStyleCopy = Object.assign({}, props.style);
    var nativeNode = this.getNativeNode();

    if (appendType !== TREE) {
      if (childMounter) {
        childMounter(nativeNode, parent);
      } else {
        _host.default.driver.appendChild(nativeNode, parent);
      }
    }

    if (this._currentElement && this._currentElement.ref) {
      _ref.default.attach(this._currentElement._owner, this._currentElement.ref, this);
    } // Process children


    var children = props.children;

    if (children != null) {
      this.mountChildren(children, context);
    }

    if (appendType === TREE) {
      if (childMounter) {
        childMounter(nativeNode, parent);
      } else {
        _host.default.driver.appendChild(nativeNode, parent);
      }
    }

    if (process.env.NODE_ENV !== 'production') {
      _host.default.hook.Reconciler.mountComponent(this);
    }

    return instance;
  };

  _proto.mountChildren = function mountChildren(children, context) {
    var _this = this;

    if (!Array.isArray(children)) {
      children = [children];
    }

    var renderedChildren = this._renderedChildren = {};
    var nativeNode = this.getNativeNode();
    var renderedChildrenImage = children.map(function (element, index) {
      var renderedChild = (0, _instantiateComponent.default)(element);
      var name = (0, _getElementKeyName.default)(renderedChildren, element, index);
      renderedChildren[name] = renderedChild;
      renderedChild._mountIndex = index; // Mount

      var mountImage = renderedChild.mountComponent(nativeNode, _this._instance, context);
      return mountImage;
    });
    return renderedChildrenImage;
  };

  _proto.unmountChildren = function unmountChildren(notRemoveChild) {
    var renderedChildren = this._renderedChildren;

    if (renderedChildren) {
      for (var name in renderedChildren) {
        var renderedChild = renderedChildren[name];
        renderedChild.unmountComponent(notRemoveChild);
      }

      this._renderedChildren = null;
    }
  };

  _proto.unmountComponent = function unmountComponent(notRemoveChild) {
    if (this._nativeNode) {
      var ref = this._currentElement.ref;

      if (ref) {
        _ref.default.detach(this._currentElement._owner, ref, this);
      }

      _instance.default.remove(this._nativeNode);

      if (!notRemoveChild) {
        _host.default.driver.removeChild(this._nativeNode, this._parent);
      }
    }

    this.unmountChildren(notRemoveChild);

    if (process.env.NODE_ENV !== 'production') {
      _host.default.hook.Reconciler.unmountComponent(this);
    }

    this._currentElement = null;
    this._nativeNode = null;
    this._parent = null;
    this._parentInstance = null;
    this._context = null;
    this._instance = null;
    this._prevStyleCopy = null;
  };

  _proto.updateComponent = function updateComponent(prevElement, nextElement, prevContext, nextContext) {
    // Replace current element
    this._currentElement = nextElement;

    _ref.default.update(prevElement, nextElement, this);

    var prevProps = prevElement.props;
    var nextProps = nextElement.props;
    this.updateProperties(prevProps, nextProps);
    this.updateChildren(nextProps.children, nextContext);

    if (process.env.NODE_ENV !== 'production') {
      _host.default.hook.Reconciler.receiveComponent(this);
    }
  };

  _proto.updateProperties = function updateProperties(prevProps, nextProps) {
    var propKey;
    var styleName;
    var styleUpdates;
    var driver = _host.default.driver;
    var nativeNode = this.getNativeNode();

    for (propKey in prevProps) {
      if (propKey === CHILDREN || nextProps.hasOwnProperty(propKey) || !prevProps.hasOwnProperty(propKey) || prevProps[propKey] == null) {
        continue;
      }

      if (propKey === STYLE) {
        var lastStyle = this._prevStyleCopy;

        for (styleName in lastStyle) {
          if (lastStyle.hasOwnProperty(styleName)) {
            styleUpdates = styleUpdates || {};
            styleUpdates[styleName] = '';
          }
        }

        this._prevStyleCopy = null;
      } else if (EVENT_PREFIX_REGEXP.test(propKey)) {
        if (typeof prevProps[propKey] === 'function') {
          driver.removeEventListener(nativeNode, propKey.slice(2).toLowerCase(), prevProps[propKey]);
        }
      } else {
        driver.removeAttribute(nativeNode, propKey, prevProps[propKey]);
      }
    }

    for (propKey in nextProps) {
      var nextProp = nextProps[propKey];
      var prevProp = propKey === STYLE ? this._prevStyleCopy : prevProps != null ? prevProps[propKey] : undefined;

      if (propKey === CHILDREN || !nextProps.hasOwnProperty(propKey) || nextProp === prevProp || nextProp == null && prevProp == null) {
        continue;
      } // Update style


      if (propKey === STYLE) {
        if (nextProp) {
          // Clone property
          nextProp = this._prevStyleCopy = Object.assign({}, nextProp);
        } else {
          this._prevStyleCopy = null;
        }

        if (prevProp != null) {
          // Unset styles on `prevProp` but not on `nextProp`.
          for (styleName in prevProp) {
            if (prevProp.hasOwnProperty(styleName) && (!nextProp || !nextProp.hasOwnProperty(styleName))) {
              styleUpdates = styleUpdates || {};
              styleUpdates[styleName] = '';
            }
          } // Update styles that changed since `prevProp`.


          for (styleName in nextProp) {
            if (nextProp.hasOwnProperty(styleName) && prevProp[styleName] !== nextProp[styleName]) {
              styleUpdates = styleUpdates || {};
              styleUpdates[styleName] = nextProp[styleName];
            }
          }
        } else {
          // Assign next prop when prev style is null
          styleUpdates = nextProp;
        }
      } else if (EVENT_PREFIX_REGEXP.test(propKey)) {
        // Update event binding
        var eventName = propKey.slice(2).toLowerCase();

        if (typeof prevProp === 'function') {
          driver.removeEventListener(nativeNode, eventName, prevProp, nextProps);
        }

        if (typeof nextProp === 'function') {
          driver.addEventListener(nativeNode, eventName, nextProp, nextProps);
        }
      } else {
        // Update other property
        var payload = {};
        payload[propKey] = nextProp;

        if (nextProp != null) {
          driver.setAttribute(nativeNode, propKey, nextProp);
        } else {
          driver.removeAttribute(nativeNode, propKey, prevProps[propKey]);
        }

        if (process.env.NODE_ENV !== 'production') {
          _host.default.measurer && _host.default.measurer.recordOperation({
            instanceID: this._mountID,
            type: 'update attribute',
            payload: payload
          });
        }
      }
    }

    if (styleUpdates) {
      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.recordOperation({
          instanceID: this._mountID,
          type: 'update style',
          payload: styleUpdates
        });
      }

      driver.setStyles(nativeNode, styleUpdates);
    }
  };

  _proto.updateChildren = function updateChildren(nextChildrenElements, context) {
    // prev rendered children
    var prevChildren = this._renderedChildren;
    var driver = _host.default.driver;

    if (nextChildrenElements == null && prevChildren == null) {
      return;
    }

    var nextChildren = {};

    if (nextChildrenElements != null) {
      if (!Array.isArray(nextChildrenElements)) {
        nextChildrenElements = [nextChildrenElements];
      } // Update next children elements


      for (var index = 0, length = nextChildrenElements.length; index < length; index++) {
        var nextElement = nextChildrenElements[index];
        var name = (0, _getElementKeyName.default)(nextChildren, nextElement, index);
        var prevChild = prevChildren && prevChildren[name];
        var prevElement = prevChild && prevChild._currentElement;
        var prevContext = prevChild && prevChild._context; // Try to update between the two of some name that has some element type,
        // and move child in next children loop if need

        if (prevChild != null && (0, _shouldUpdateComponent.default)(prevElement, nextElement)) {
          if (prevElement !== nextElement || prevContext !== context) {
            // Pass the same context when updating chidren
            prevChild.updateComponent(prevElement, nextElement, context, context);
          }

          nextChildren[name] = prevChild;
        } else {
          // Unmount the prevChild when some name with nextChild but different element type,
          // and move child node in next children loop
          if (prevChild) {
            prevChild._unmount = true;
          } // The child must be instantiated before it's mounted.


          nextChildren[name] = (0, _instantiateComponent.default)(nextElement);
        }
      }
    }

    var prevFirstChild;
    var prevFirstNativeNode;
    var shouldUnmountPrevFirstChild; // Unmount children that are no longer present.

    if (prevChildren != null) {
      for (var _name in prevChildren) {
        if (!prevChildren.hasOwnProperty(_name)) {
          continue;
        }

        var _prevChild = prevChildren[_name];
        var shouldUnmount = _prevChild._unmount || !nextChildren[_name]; // Store old first child ref for append node ahead and maybe delay remove it

        if (!prevFirstChild) {
          shouldUnmountPrevFirstChild = shouldUnmount;
          prevFirstChild = _prevChild;
          prevFirstNativeNode = prevFirstChild.getNativeNode();

          if (Array.isArray(prevFirstNativeNode)) {
            prevFirstNativeNode = prevFirstNativeNode[0];
          }
        } else if (shouldUnmount) {
          _prevChild.unmountComponent();
        }
      }
    }

    if (nextChildren != null) {
      var insertNodes = function insertNodes(nativeNodes, parent) {
        // The nativeNodes maybe fragment, so convert to array type
        if (!Array.isArray(nativeNodes)) {
          nativeNodes = [nativeNodes];
        }

        if (lastPlacedNode) {
          // Should reverse order when insert new child after lastPlacedNode:
          // [lastPlacedNode, *newChild1, *newChild2]
          for (var i = nativeNodes.length - 1; i >= 0; i--) {
            driver.insertAfter(nativeNodes[i], lastPlacedNode);
          }
        } else if (prevFirstNativeNode) {
          // [*newChild1, *newChild2, prevFirstNativeNode]
          for (var _i = 0; _i < nativeNodes.length; _i++) {
            driver.insertBefore(nativeNodes[_i], prevFirstNativeNode);
          }
        } else if (parent) {
          // [*newChild1, *newChild2]
          for (var _i2 = 0; _i2 < nativeNodes.length; _i2++) {
            driver.appendChild(nativeNodes[_i2], parent);
          }
        }
      };

      // `nextIndex` will increment for each child in `nextChildren`
      var nextIndex = 0;
      var lastPlacedNode = null;
      var nextNativeNode = [];

      for (var _name2 in nextChildren) {
        if (!nextChildren.hasOwnProperty(_name2)) {
          continue;
        }

        var nextChild = nextChildren[_name2];

        var _prevChild2 = prevChildren && prevChildren[_name2]; // Try to move the some key prevChild but current not at the some position


        if (_prevChild2 === nextChild) {
          var prevChildNativeNode = _prevChild2.getNativeNode();

          if (_prevChild2._mountIndex !== nextIndex) {
            insertNodes(prevChildNativeNode);
          }
        } else {
          // Mount nextChild that in prevChildren there has no some name
          var parent = this.getNativeNode(); // Fragment extended native component, so if parent is fragment should get this._parent

          if (Array.isArray(parent)) {
            parent = this._parent;
          }

          nextChild.mountComponent(parent, this._instance, context, insertNodes // Insert nodes mounter
          );
        } // Update to the latest mount order


        nextChild._mountIndex = nextIndex++; // Get the last child

        lastPlacedNode = nextChild.getNativeNode(); // Push to nextNativeNode

        nextNativeNode = nextNativeNode.concat(lastPlacedNode);

        if (Array.isArray(lastPlacedNode)) {
          lastPlacedNode = lastPlacedNode[lastPlacedNode.length - 1];
        }
      } // Sync update native refs


      if (Array.isArray(this._nativeNode)) {
        // Clear all and push the new array
        this._nativeNode.splice(0, this._nativeNode.length);

        for (var i = 0; i < nextNativeNode.length; i++) {
          this._nativeNode.push(nextNativeNode[i]);
        }
      }
    }

    if (shouldUnmountPrevFirstChild) {
      prevFirstChild.unmountComponent();
    }

    this._renderedChildren = nextChildren;
  };

  _proto.getNativeNode = function getNativeNode() {
    if (this._nativeNode == null) {
      this._nativeNode = _host.default.driver.createElement(this._instance);

      _instance.default.set(this._nativeNode, this._instance);
    }

    return this._nativeNode;
  };

  _proto.getPublicInstance = function getPublicInstance() {
    return this.getNativeNode();
  };

  _proto.getName = function getName() {
    return this._currentElement.type;
  };

  return NativeComponent;
}();

var _default = NativeComponent;
exports.default = _default;